/*
 *
 *           MULti Core Operating System - Tiny / Multilayer Kernel
 *                      Device header for RA8 AGT
 *
 *           Copyright (c) 2025 SPIRAL-TECH Co.,ltd All Rights Reserved.
 *
 *  File name       dv_ra8_agt.h
 *
 *  Feb/02/2025     Created.
 *
 */

#ifndef _DV_RA8_AGT_H_
#define _DV_RA8_AGT_H_
#ifdef __cplusplus
extern "C" {
#endif

/* Base Address */

#define AGT0_BASE       0x40221000      /* AGT0 Base address */
#define AGT1_BASE       0x40221100      /* AGT1 Base address */

#define NS_BASE_OFFSET  0x10000000      /* NonSecure peripheral offset address */


/* Evento number */

#define EVENT_AGT0_BASE         0x046
#define EVENT_AGT1_BASE         0x049

#define EVENT_AGTI_OFFSET       0
#define EVENT_AGTCMAI_OFFSET    1
#define EVENT_AGTCMBI_OFFSET    2


/* registers */

typedef struct {
    UH  AGT;              /* AGT Counter register                              */
    UH  AGTCMA;           /* AGT Compare Match A register                      */
    UH  AGTCMB;           /* AGT Compare Match B register                      */
    UH  reserved;
    UB  AGTCR;            /* AGT Control register                              */
    UB  AGTMR1;           /* AGT Mode register 1                               */
    UB  AGTMR2;           /* AGT Mode register 2                               */
    UB  reserved1;
    UB  AGTIOC;           /* AGT I/O Control register                          */
    UB  AGTISR;           /* AGT Event Pin Select register                     */
    UB  AGTCMSR;          /* AGT Compare Match Function Select register        */
    UB  AGTIOSEL;         /* AGT Pin Select register                           */
} T_RA8_AGT; 


/* Port access structure */

#ifdef __BUILD_SECURE__
#define REG_AGT0    (*(volatile T_RA8_AGT *)AGT0_BASE)
#define REG_AGT1    (*(volatile T_RA8_AGT *)AGT1_BASE)
#else
#define REG_AGT0    (*(volatile T_RA8_AGT *)(AGT0_BASE | NS_BASE_OFFSET))
#define REG_AGT1    (*(volatile T_RA8_AGT *)(AGT1_BASE | NS_BASE_OFFSET))
#endif

#ifdef __cplusplus
}
#endif
#endif /* _DV_RA8_AGT_H_ */
