/*
 *
 *           MULti Core Operating System - Tiny / Multilayer Kernel
 *                      Device header for RA8 CPSCU
 *
 *           Copyright (c) 2025 SPIRAL-TECH Co.,ltd All Rights Reserved.
 *
 *  File name       dv_ra8_cpscu.h
 *
 *  Feb/02/2025     Created.
 *
 */

#ifndef _DV_RA8_CPSCU_H_
#define _DV_RA8_CPSCU_H_
#ifdef __cplusplus
extern "C" {
#endif

/* Base Address */

#define CPSCU_BASE      0x40008000      /* CPSCU Base address */

#define NS_BASE_OFFSET  0x10000000      /* NonSecure peripheral offset address */


/* registers */

typedef struct {
    UW    CSAR;                   /* Cache Security Attribution register       */
    UW    reserved[3];
    UW    SRAMSAR;                /* SRAM Security Attribution register        */
    UW    STBRAMSAR;              /* Standby RAM memory Security Attribution register */
    UW    reserved1[6];
    UW    DTCSAR;                 /* DTC Controller Security Attribution register */
    UW    DMACSAR;                /* DMAC Controller Security Attribution register */
    UW    reserved2[2];
    UW    ICUSARA;                /* ICU Security Attribution register A        */
    UW    ICUSARB;                /* ICU Security Attribution register B        */
    UW    ICUSARC;                /* ICU Security Attribution register C        */
    UW    ICUSARD;                /* ICU Security Attribution register D        */
    UW    ICUSARE;                /* ICU Security Attribution register E        */
    UW    ICUSARF;                /* ICU Security Attribution register F        */
    UW    reserved3[6];
    UW    ICUSARG;                /* ICU Security Attribution register G        */
    UW    ICUSARH;                /* ICU Security Attribution register H        */
    UW    ICUSARI;                /* ICU Security Attribution register I        */
    UW    reserved4[33];
    UW    BUSSARA;                /* Bus Security Attribution register A        */
    UW    BUSSARB;                /* Bus Security Attribution register B        */
    UW    reserved5[10];
    UW    MMPUSARA;               /* Master Memory Protection Unit Security Attribution register A */
    UW    MMPUSARB;               /* Master Memory Protection Unit Security Attribution register B */
    UW    reserved6[26];
    UW    DMACCHSAR;              /* DMA channel Security Attribution register  */
    UW    reserved7[3];
    UW    CPUDSAR;                /* CPU Debug Security Attribution register    */
    UW    reserved8[147];
    UW    SRAMSABAR0;             /* SRAM Security Attribute Boundary Address register 0 */
    UW    SRAMSABAR1;             /* SRAM Security Attribute Boundary Address register 1 */
    UW    reserved9[126];
    UW    TEVTRCR;                /* Trusted Event Route Control register       */
} T_RA8_CPSCU; 


/* Port access structure */

#ifdef __BUILD_SECURE__
#define REG_CPSCU   (*(volatile T_RA8_CPSCU *)CPSCU_BASE)
#else
#define REG_CPSCU   (*(volatile T_RA8_CPSCU *)(CPSCU_BASE | NS_BASE_OFFSET))
#endif


#ifdef __cplusplus
}
#endif
#endif /* _DV_RA8_CPSCU_H_ */
