/*
 *
 *           MULti Core Operating System - Tiny / Multilayer Kernel
 *                      Device header for RA8 FLASH
 *
 *           Copyright (c) 2025 SPIRAL-TECH Co.,ltd All Rights Reserved.
 *
 *  File name       dv_ra8_flash.h
 *
 *  Feb/02/2025     Created.
 *
 */

#ifndef _DV_RA8_FLASH_H_
#define _DV_RA8_FLASH_H_
#ifdef __cplusplus
extern "C" {
#endif

/* Base Address */

#define FLASH_BASE      0x4001C100      /* FLASH Base address */
#define FACI_BASE       0x4011E000      /* FACI Base address */
#define FACI_CMD_BASE   0x40100000      /* FACI CMD Base address */

#define NS_BASE_OFFSET  0x10000000      /* NonSecure peripheral offset address */


/* registers */

typedef struct {
    UH    FCACHEE;                /* Flash Cache Enable Register               */
    UH    reserved;
    UH    FCACHEIV;               /* Flash Cache Invalidate Register           */
    UH    reserved1[11];
    UB    FLWT;                   /* Flash Wait Cycle Register                 */
    UB    reserved2;
    UH    reserved3[17];
    UH    FSAR;                   /* Flash Security Attribution Register       */
} T_RA8_FLASH; 


typedef struct {
    union {
        UH    CMD16;              /* FACI Command Issuing Area (half access)   */
        UB    CMD8;               /* FACI Command Issuing Area (byte access)   */
    };
} T_RA8_FACI_CMD; 


typedef struct {
    UW  reserved[4];
    UB  FASTAT;                   /* Flash Access Status                       */
    UB  reserved1;
    UH  reserved2;
    UB  FAEINT;                   /* Flash Access Error Interrupt Enable       */
    UB  reserved3;
    UH  reserved4;
    UB  FRDYIE;                   /* Flash Ready Interrupt Enable              */
    UB  reserved5;
    UH  reserved6;
    UW  reserved7[5];
    UW  FSADDR;                   /* Flash Start Address                       */
    UW  FEADDR;                   /* Flash End Address                         */
    UW  reserved8[3];
    UH  FMEPROT;                  /* Flash P/E Mode Entry Protection Register  */
    UH  reserved9;
    UW  reserved10[12];
    UH  FBPROT0;                  /* Flash Block Protection Register           */
    UH  reserved11;
    UH  FBPROT1;                  /* Flash Block Protection for Secure Register*/
    UH  reserved12;
    UW  FSTATR;                   /* Flash Status                              */
    UH  FENTRYR;                  /* Program/Erase Mode Entry                  */
    UH  reserved13;
    UW  reserved14;
    UH  FSUINITR;                 /* Flash Sequencer Set-up Initialize         */
    UH  reserved15;
    UW  reserved16[4];
    UH  FCMDR;                    /* Flash Sequencer Command                   */
    UH  reserved17;
    UW  reserved18[11];
    UB  FBCCNT;                   /* Blank Check Control                       */
    UB  reserved19;
    UH  reserved20;
    UB  FBCSTAT;                  /* Blank Check Status                        */
    UB  reserved21;
    UH  reserved22;
    UW  FPSADDR;                  /* Programmed Area Start Address             */
    UW  FAWMON;                   /* Flash Access Window Monitor               */
    UH  FCPSR;                    /* FCU Process Switch                        */
    UH  reserved23;
    UH  FPCKAR;                   /* Flash Sequencer Processing Clock Frequency Notification */
    UH  reserved24;
    UH  FSUACR;                   /* Flash Start-Up Area Control Register      */
} T_RA8_FACI; 


/* Port access structure */

#ifdef __BUILD_SECURE__
#define REG_FLASH     (*(volatile T_RA8_FLASH *)FLASH_BASE)
#define REG_FACI      (*(volatile T_RA8_FACI *)FACI_BASE)
#define REG_FACI_CMD  (*(volatile T_RA8_FACI_CMD *)FACI_CMD_BASE)
#else
#define REG_FLASH     (*(volatile T_RA8_FLASH *)(FLASH_BASE | NS_BASE_OFFSET))
#define REG_FACI      (*(volatile T_RA8_FACI *)(FACI_BASE | NS_BASE_OFFSET))
#define REG_FACI_CMD  (*(volatile T_RA8_FACI_CMD *)(FACI_CMD_BASE | NS_BASE_OFFSET))
#endif


#ifdef __cplusplus
}
#endif
#endif /* _DV_RA8_FLASH_H_ */
