/*
 *
 *           MULti Core Operating System - Tiny / Multilayer Kernel
 *                      Device header for RA8 ICU
 *
 *           Copyright (c) 2025 SPIRAL-TECH Co.,ltd All Rights Reserved.
 *
 *  File name       dv_ra8_icu.h
 *
 *  Feb/02/2025     Created.
 *
 */

#ifndef _DV_RA8_ICU_H_
#define _DV_RA8_ICU_H_
#ifdef __cplusplus
extern "C" {
#endif

/* Base Address */

#define ICU_BASE        0x40006000      /* ICU Base address */

#define NS_BASE_OFFSET  0x10000000      /* NonSecure peripheral offset address */

#if defined(_DV_RA8M1_H_)
#define EVENT_MAX     0x19C
#elif defined(_DV_RA8D1_H_)
#define EVENT_MAX     0x1DA
#else
#error "I/O definition file is undefined."
#endif


/* registers */

typedef struct {
    UB    IRQCR[16];              /* IRQ Control register [0..15]              */
    UB    NMICR;                  /* NMI Pin Interrupt Control register        */
    UB    reserved;
    UH    reserved1;
    UW    reserved2[6143];
    UB    SWIRQ_S;                /* Software Interrupt Request register for Secure Interrupt */
    UB    reserved3;
    UH    reserved4;
    UW    reserved5[3];
    UB    SWIRQ_NS;               /* Software Interrupt Request register for Non-secure Interrupt */
    UB    reserved6;
    UH    reserved7;
    UW    reserved8[15];
    UH    IENMIER;                /* Integrated Error NMI Interrupt Enable Registe for CPU */
    UH    reserved9;
    UW    reserved10[39];
    UH    NMIER;                  /* Non-Maskable Interrupt Enable register    */
    UH    reserved11;
    UW    reserved12[3];
    UH    NMICLR;                 /* Non-Maskable Interrupt Status Clear register */
    UH    reserved13;
    UW    reserved14[3];
    UH    NMISR;                  /* Non-Maskable Interrupt Status register    */
    UH    reserved15;
    UW    reserved16[31];
    UW    WUPEN;                  /* Wake Up Interrupt Enable register         */
    UW    WUPEN1;                 /* Wake Up Interrupt Enable register 1       */
    UW    reserved17[86];
    UW    IELSR[96];              /* ICU Event Link Setting register [0..95]   */
} T_RA8_ICU; 


/* Port access structure */

#ifdef __BUILD_SECURE__
#define REG_ICU     (*(volatile T_RA8_ICU *)ICU_BASE)
#else
#define REG_ICU     (*(volatile T_RA8_ICU *)(ICU_BASE | NS_BASE_OFFSET))
#endif

#ifdef __cplusplus
}
#endif
#endif /* _DV_RA8_ICU_H_ */
