/*
 *
 *           MULti Core Operating System - Tiny / Multilayer Kernel
 *                      Device header for RA8 PFS&PORT
 *
 *           Copyright (c) 2025 SPIRAL-TECH Co.,ltd All Rights Reserved.
 *
 *  File name       dv_ra8_pfs_port.h
 *
 *  Feb/02/2025     Created.
 *
 */

#ifndef _DV_RA8_PFS_PORT_H_
#define _DV_RA8_PFS_PORT_H_
#ifdef __cplusplus
extern "C" {
#endif

/* Base Address */

#define PORT0_BASE      0x40400000      /* PORTCR0 Base address */
#define PORT1_BASE      0x40400020      /* PORTCR1 Base address */
#define PORT2_BASE      0x40400040      /* PORTCR2 Base address */
#define PORT3_BASE      0x40400060      /* PORTCR3 Base address */
#define PORT4_BASE      0x40400080      /* PORTCR4 Base address */
#define PORT5_BASE      0x404000A0      /* PORTCR5 Base address */
#define PORT6_BASE      0x404000C0      /* PORTCR6 Base address */
#define PORT7_BASE      0x404000E0      /* PORTCR7 Base address */
#define PORT8_BASE      0x40400100      /* PORTCR8 Base address */
#define PORT9_BASE      0x40400120      /* PORTCR9 Base address */
#define PORTA_BASE      0x40400140      /* PORTCRA Base address */
#define PORTB_BASE      0x40400160      /* PORTCRB Base address */

#define PFS0_BASE       0x40400800      /* PFS0 Base address */
#define PFS1_BASE       0x40400840      /* PFS1 Base address */
#define PFS2_BASE       0x40400880      /* PFS2 Base address */
#define PFS3_BASE       0x404008C0      /* PFS3 Base address */
#define PFS4_BASE       0x40400900      /* PFS4 Base address */
#define PFS5_BASE       0x40400940      /* PFS5 Base address */
#define PFS6_BASE       0x40400980      /* PFS6 Base address */
#define PFS7_BASE       0x404009C0      /* PFS7 Base address */
#define PFS8_BASE       0x40400A00      /* PFS8 Base address */
#define PFS9_BASE       0x40400A40      /* PFS9 Base address */
#define PFSA_BASE       0x40400A80      /* PFSA Base address */
#define PFSB_BASE       0x40400AC0      /* PFSB Base address */

#define PFENET_BASE     0x40400D00      /* Ethernet and Write Protect address */
#define PWPR_NS_BASE    0x40400D0C      /* Write Protect for Non-secure address */
#define PWPR_S_BASE     0x40400D14      /* Write Protect for Secure address */

#define SAR_BASE        0x40400D30      /* SAR Base address */

#define NS_BASE_OFFSET  0x10000000      /* NonSecure peripheral offset address */


/* registers */

typedef struct {
    union {
        UW    WORD;               /* Port Control Register 1                   */
        struct {
            UH    PDR;            /* Data direction register                   */
            UH    PODR;           /* Output data register                      */
        };
    } PCNTR1;
    union {
        UW    WORD;               /* Port Control Register 2                   */
        struct {
            UH    PIDR;           /* Input data register                       */
            UH    EIDR;           /* Event input data register                 */
        };
    } PCNTR2;
    union {
        UW    WORD;               /* Port Control Register 3                   */
        struct {
            UH    POSR;           /* Output reset register                     */
            UH    PORR;           /* Output set register                       */
        };
    } PCNTR3;
    union {
        UW    WORD;               /* Port Control Register 4                   */
        struct {
            UH    EOSR;           /* Event output reset register               */
            UH    EORR;           /* Event output set register                 */
        };
    } PCNTR4;
} T_RA8_PORT; 

typedef struct {
    union {
        UW    PnmPFS;             /* Port mn Pin Function Select register      */
        UH    PnmPFS_HA;
        UB    PnmPFS_BY;
    };
} T_RA8_PFS;

typedef struct {
    UH    PmSAR;                  /* Port Security Attribution register        */
    UH    reserved4;
} T_RA8_SAR; 


/* Port access structure */

#ifdef __BUILD_SECURE__
#define REG_PORT0   (*(volatile T_RA8_PORT *)PORT0_BASE)
#define REG_PORT1   (*(volatile T_RA8_PORT *)PORT1_BASE)
#define REG_PORT2   (*(volatile T_RA8_PORT *)PORT2_BASE)
#define REG_PORT3   (*(volatile T_RA8_PORT *)PORT3_BASE)
#define REG_PORT4   (*(volatile T_RA8_PORT *)PORT4_BASE)
#define REG_PORT5   (*(volatile T_RA8_PORT *)PORT5_BASE)
#define REG_PORT6   (*(volatile T_RA8_PORT *)PORT6_BASE)
#define REG_PORT7   (*(volatile T_RA8_PORT *)PORT7_BASE)
#define REG_PORT8   (*(volatile T_RA8_PORT *)PORT8_BASE)
#define REG_PORT9   (*(volatile T_RA8_PORT *)PORT9_BASE)
#define REG_PORTA   (*(volatile T_RA8_PORT *)PORTA_BASE)
#define REG_PORTB   (*(volatile T_RA8_PORT *)PORTB_BASE)

#define REG_PFS0    ( (volatile T_RA8_PFS *)PFS0_BASE)
#define REG_PFS1    ( (volatile T_RA8_PFS *)PFS1_BASE)
#define REG_PFS2    ( (volatile T_RA8_PFS *)PFS2_BASE)
#define REG_PFS3    ( (volatile T_RA8_PFS *)PFS3_BASE)
#define REG_PFS4    ( (volatile T_RA8_PFS *)PFS4_BASE)
#define REG_PFS5    ( (volatile T_RA8_PFS *)PFS5_BASE)
#define REG_PFS6    ( (volatile T_RA8_PFS *)PFS6_BASE)
#define REG_PFS7    ( (volatile T_RA8_PFS *)PFS7_BASE)
#define REG_PFS8    ( (volatile T_RA8_PFS *)PFS8_BASE)
#define REG_PFS9    ( (volatile T_RA8_PFS *)PFS9_BASE)
#define REG_PFSA    ( (volatile T_RA8_PFS *)PFSA_BASE)
#define REG_PFSB    ( (volatile T_RA8_PFS *)PFSB_BASE)

#define REG_PFENET  (*(volatile UB *)PFENET_BASE)
#define REG_PWPR_S  (*(volatile UB *)PWPR_S_BASE)

#define REG_SAR     ( (volatile T_RA8_SAR *)SAR_BASE)

#else
#define REG_PORT0   (*(volatile T_RA8_PORT *)(PORT0_BASE | NS_BASE_OFFSET))
#define REG_PORT1   (*(volatile T_RA8_PORT *)(PORT1_BASE | NS_BASE_OFFSET))
#define REG_PORT2   (*(volatile T_RA8_PORT *)(PORT2_BASE | NS_BASE_OFFSET))
#define REG_PORT3   (*(volatile T_RA8_PORT *)(PORT3_BASE | NS_BASE_OFFSET))
#define REG_PORT4   (*(volatile T_RA8_PORT *)(PORT4_BASE | NS_BASE_OFFSET))
#define REG_PORT5   (*(volatile T_RA8_PORT *)(PORT5_BASE | NS_BASE_OFFSET))
#define REG_PORT6   (*(volatile T_RA8_PORT *)(PORT6_BASE | NS_BASE_OFFSET))
#define REG_PORT7   (*(volatile T_RA8_PORT *)(PORT7_BASE | NS_BASE_OFFSET))
#define REG_PORT8   (*(volatile T_RA8_PORT *)(PORT8_BASE | NS_BASE_OFFSET))
#define REG_PORT9   (*(volatile T_RA8_PORT *)(PORT9_BASE | NS_BASE_OFFSET))
#define REG_PORTA   (*(volatile T_RA8_PORT *)(PORTA_BASE | NS_BASE_OFFSET))
#define REG_PORTB   (*(volatile T_RA8_PORT *)(PORTB_BASE | NS_BASE_OFFSET))

#define REG_PFS0    ( (volatile T_RA8_PFS *)(PFS0_BASE | NS_BASE_OFFSET))
#define REG_PFS1    ( (volatile T_RA8_PFS *)(PFS1_BASE | NS_BASE_OFFSET))
#define REG_PFS2    ( (volatile T_RA8_PFS *)(PFS2_BASE | NS_BASE_OFFSET))
#define REG_PFS3    ( (volatile T_RA8_PFS *)(PFS3_BASE | NS_BASE_OFFSET))
#define REG_PFS4    ( (volatile T_RA8_PFS *)(PFS4_BASE | NS_BASE_OFFSET))
#define REG_PFS5    ( (volatile T_RA8_PFS *)(PFS5_BASE | NS_BASE_OFFSET))
#define REG_PFS6    ( (volatile T_RA8_PFS *)(PFS6_BASE | NS_BASE_OFFSET))
#define REG_PFS7    ( (volatile T_RA8_PFS *)(PFS7_BASE | NS_BASE_OFFSET))
#define REG_PFS8    ( (volatile T_RA8_PFS *)(PFS8_BASE | NS_BASE_OFFSET))
#define REG_PFS9    ( (volatile T_RA8_PFS *)(PFS9_BASE | NS_BASE_OFFSET))
#define REG_PFSA    ( (volatile T_RA8_PFS *)(PFSA_BASE | NS_BASE_OFFSET))
#define REG_PFSB    ( (volatile T_RA8_PFS *)(PFSB_BASE | NS_BASE_OFFSET))

#define REG_PFENET  (*(volatile UB *)(PFENET_BASE | NS_BASE_OFFSET))
#define REG_PWPR_NS (*(volatile UB *)(PWPR_NS_BASE | NS_BASE_OFFSET))

#define REG_SAR     ( (volatile T_RA8_SAR *)(SAR_BASE | NS_BASE_OFFSET))
#endif


#ifdef __cplusplus
}
#endif
#endif /* _DV_RA8_PFS_PORT_H_ */
