/*
 *
 *           MULti Core Operating System - Tiny / Multilayer Kernel
 *                      Device header for RA8 PSCU
 *
 *           Copyright (c) 2025 SPIRAL-TECH Co.,ltd All Rights Reserved.
 *
 *  File name       dv_ra8_pscu.h
 *
 *  Feb/02/2025     Created.
 *
 */

#ifndef _DV_RA8_PSCU_H_
#define _DV_RA8_PSCU_H_
#ifdef __cplusplus
extern "C" {
#endif

/* Base Address */

#define PSCU_BASE       0x40204000      /* PSCU Base address */

#define NS_BASE_OFFSET  0x10000000      /* NonSecure peripheral offset address */


/* registers */

typedef struct {
    UW    reserved;
    UW    PSARB;                  /* Peripheral Security Attribution register B */
    UW    PSARC;                  /* Peripheral Security Attribution register C */
    UW    PSARD;                  /* Peripheral Security Attribution register D */
    UW    PSARE;                  /* Peripheral Security Attribution register E */
    UW    MSSAR;                  /* Module Stop Security Attribution register  */
    UW    reserved1;
    UW    PPARB;                  /* Peripheral Privilege Attribution register B */
    UW    PPARC;                  /* Peripheral Privilege Attribution register C */
    UW    PPARD;                  /* Peripheral Privilege Attribution register D */
    UW    PPARE;                  /* Peripheral Privilege Attribution register E */
    UW    MSPAR;                  /* Module Stop Privilege Attribution register  */
    UW    CFSAMONA;               /* Code Flash Security Attribution Monitor register A */
    UW    DFSAMON;                /* Data Flash Security Attribution Monitor register */
    UW    DLMMON;                 /* Device Lifecycle Management State Monitor register */
} T_RA8_PSCU; 


/* Port access structure */

#ifdef __BUILD_SECURE__
#define REG_PSCU    (*(volatile T_RA8_PSCU *)PSCU_BASE)
#else
#define REG_PSCU    (*(volatile T_RA8_PSCU *)(PSCU_BASE | NS_BASE_OFFSET))
#endif

#ifdef __cplusplus
}
#endif
#endif /* _DV_RA8_PSCU_H_ */
