/*
 *
 *           MULti Core Operating System - Tiny / Multilayer Kernel
 *                      Device header for RA8 SRAM
 *
 *           Copyright (c) 2025 SPIRAL-TECH Co.,ltd All Rights Reserved.
 *
 *  File name       dv_ra8_sram.h
 *
 *  Feb/02/2025     Created.
 *
 */

#ifndef _DV_RA8_SRAM_H_
#define _DV_RA8_SRAM_H_
#ifdef __cplusplus
extern "C" {
#endif

/* Base Address */

#define SRAM_BASE       0x40002000      /* SRAM Base address */

#define NS_BASE_OFFSET  0x10000000      /* NonSecure peripheral offset address */


/* registers */

typedef struct {
    UH  SRAMPRCR_S;       /* SRAM Protection Control Register for Secure       */
    UH  reserved;
    UH  SRAMPRCR_NS;      /* SRAM Protection Control Register for Non-Secure   */
    UH  reserved1;
    UB  SRAMWTSC;         /* SRAM Wait State Control Register                  */
    UB  reserved2;
    UH  reserved3;
    UW  reserved4;
    UB  SRAMCR0;          /* SRAM Control Register 0                           */
    UB  reserved5;
    UH  reserved6;
    UB  SRAMCR1;          /* SRAM Control Register 1                           */
    UB  reserved7;
    UH  reserved8;
    UW  reserved9[6];
    UB  SRAMECCRGN0;      /* SRAM0 ECC Region Control Register                 */
    UB  reserved10;
    UH  reserved11;
    UW  reserved12[3];
    UH  SRAMESR;          /* SRAM Error Status Register                        */
    UH  reserved13;
    UW  reserved14;
    UH  SRAMESCLR;        /* SRAM Error Status Clear Register                  */
    UH  reserved15;
    UW  reserved16;
    UW  SRAMEAR0;         /* SRAM Error Address Register                       */
    UW  SRAMEAR1;         /* SRAM Error Address Register                       */
    UW  SRAMEAR2;         /* SRAM Error Address Register                       */
    UB  STBRAMCR;         /* Standby SRAM Control Register                     */
    UB  reserved18;
    UH  reserved19;
    UW  reserved20[15];
    UW  STBRAMEAR;        /* Standby SRAM Error Address Register               */
} T_RA8_SRAM; 


/* Port access structure */

#ifdef __BUILD_SECURE__
#define REG_SRAM      (*(volatile T_RA8_SRAM *)SRAM_BASE)
#else
#define REG_SRAM      (*(volatile T_RA8_SRAM *)(SRAM_BASE | NS_BASE_OFFSET))
#endif


#ifdef __cplusplus
}
#endif
#endif /* _DV_RA8_SRAM_H_ */
