/*
 *
 *           MULti Core Operating System - Tiny / Multilayer Kernel
 *                      Device header for RA8 SYSC
 *
 *           Copyright (c) 2025 SPIRAL-TECH Co.,ltd All Rights Reserved.
 *
 *  File name       dv_ra8_sysc.h
 *
 *  Feb/02/2025     Created.
 *
 */

#ifndef _DV_RA8_SYSC_H_
#define _DV_RA8_SYSC_H_
#ifdef __cplusplus
extern "C" {
#endif

/* Base Address */

#define SYSC_BASE       0x4001E000      /* SYSC Base address */

#define NS_BASE_OFFSET  0x10000000      /* NonSecure peripheral offset address */


/* Registers */

typedef struct {
    UW    reserved[3];
    UB    SBYCR;                  /* Standby Control register                 */
    UB    reserved1;
    UB    SSCR2;                  /* Software Standby Control register 2      */
    UB    reserved2;
    UB    FLSCR;                  /* Flash Standby Control register           */
    UB    reserved3;
    UH    reserved4;
    UW    reserved5[2];
    UW    MSTPCRA;                /* Module Stop Control register A           */
    UW    SCKDIVCR;               /* System Clock Division Control register   */
    UB    SCKDIVCR2;              /* System Clock Division Control register 2 */
    UB    reserved6;
    UB    SCKSCR;                 /* System Clock Source Control register     */
    UB    reserved7;
    UH    PLLCCR;                 /* PLL Clock Control register               */
    UB    PLLCR;                  /* PLL Control register                     */
    UB    reserved8;
    UW    reserved9;
    UB    BCKCR;                  /* External Bus Clock Control register      */
    UB    reserved10;
    UB    MOSCCR;                 /* Main Clock Oscillator Control register   */
    UB    reserved11;
    UH    reserved12;
    UB    HOCOCR;                 /* High-Speed On-Chip Oscillator Control register */
    UB    reserved13;
    UB    MOCOCR;                 /* Middle-Speed On-Chip Oscillator Control register */
    UB    FLLCR1;                 /* FLL Control register 1                   */
    UH    FLLCR2;                 /* FLL Control register 2                   */
    UB    OSCSF;                  /* Oscillation Stabilization Flag register  */
    UB    reserved14;
    UB    CKOCR;                  /* Clock Out Control register               */
    UB    TRCKCR;                 /* Trace Clock Control register             */
    UB    OSTDCR;                 /* Oscillation Stop Detection Control register */
    UB    OSTDSR;                 /* Oscillation Stop Detection Status register */
    UB    reserved15;
    UB    OSCMONR;                /* Oscillator Monitor register              */
    UW    reserved16;
    UH    PLL2CCR;                /* PLL2 Clock Control register              */
    UB    PLL2CR;                 /* PLL2 Control register                    */
    UB    reserved17;
    UH    PLLCCR2;                /* PLL Clock Control register 2             */
    UH    PLL2CCR2;               /* PLL2 Clock Control register 2            */
    UH    reserved18;
    UB    EBCKOCR;                /* External Bus Clock Output Control register */
    UB    SDCKOCR;                /* SDRAM Clock Output Control register      */
    UB    SCICKDIVCR;             /* SCI clock Division control register      */
    UB    SCICKCR;                /* SCI clock control register               */
    UB    SPICKDIVCR;             /* SPI clock Division control register      */
    UB    SPICKCR;                /* SPI clock control register               */
    UH    reserved19;
    UB    ADCCKDIVCR;             /* ADC clock Division control register      */
    UB    ADCCKCR;                /* ADC clock control register               */
    UB    GPTCKDIVCR;             /* GPT clock Division control register      */
    UB    GPTCKCR;                /* GPT clock control register               */
    UB    LCDCKDIVCR;             /* LCD clock Division control register      */
    UB    LCDCKCR;                /* LCD clock control register               */
    UB    reserved20;
    UB    MOCOUTCR;               /* MOCO User Trimming Control register      */
    UB    HOCOUTCR;               /* HOCO User Trimming Control register      */
    UB    reserved21;
    UW    reserved22[2];
    UB    USBCKDIVCR;             /* USB clock Division control register      */
    UB    OCTACKDIVCR;            /* Octal-SPI clock Division control register */
    UB    CANFDCKDIVCR;           /* CANFD Core clock Division control register */
    UB    USB60CKDIVCR;           /* USB60 clock Division control register    */
    UB    I3CCKDIVCR;             /* I3C clock Division control register      */
    UB    reserved23;
    UH    reserved24;
    UB    USBCKCR;                /* USB clock control register               */
    UB    OCTACKCR;               /* Octal-SPI clock control register         */
    UB    CANFDCKCR;              /* CANFD Core clock control register        */
    UB    USB60CKCR;              /* USB60 clock control register             */
    UB    I3CCKCR;                /* I3C clock control register               */
    UB    reserved25;
    UH    reserved26;
    UB    MOSCSCR;                /* Main Clock Oscillator Standby Control register */
    UB    HOCOSCR;                /* High-Speed On-Chip Oscillator Standby Control */
    UH    reserved27;
    UW    reserved28[8];
    UB    OPCCR;                  /* Operating Power Control register         */
    UB    reserved29;
    UB    MOSCWTCR;               /* Main Clock Oscillator Wait Control register */
    UB    reserved30[2];
    UB    HOCOWTCR;               /* High-speed on-chip oscillator wait control register */
    UH    reserved31[2];
    UB    SOPCCR;                 /* Sub Operating Power Control register     */
    UB    reserved32;
    UW    reserved33[5];
    UW    RSTSR1;                 /* Reset Status register 1                  */
    UW    reserved34[2];
    UB    SYRACCR;                /* System register Access Control register  */
    UB    reserved35;
    UH    reserved36;
    UW    reserved37[4];
    UB    LVD1CR1;                /* Voltage Monitor 1 Circuit Control register 1 */
    UB    LVD1SR;                 /* Voltage Monitor 1 Circuit Status register */
    UB    LVD2CR1;                /* Voltage Monitor 2 Circuit Control register 1 */
    UB    LVD2SR;                 /* Voltage Monitor 2 Circuit Status register */
    UW    reserved38[3];
    UB    CRVSYSCR;               /* Clock Recovery System Control register    */
    UB    reserved39;
    UH    reserved40;
    UW    reserved41[7];
    UB    PDCTRGD;                /* Graphics Power Domain Control register    */
    UB    reserved42;
    UH    reserved43;
    UW    reserved44[11];
    UH    PDRAMSCR0;              /* SRAM power domain Standby Control register 0 */
    UB    PDRAMSCR1;              /* SRAM power domain Standby Control register 1 */
    UB    reserved45;
    UW    reserved46[155];
    UH    VBRSABAR;               /* VBATT Backup register Security Attribute Boundary  */
    UH    reserved47;
    UH    VBRPABARS;              /* VBATT Backup register Privilege Attribute Boundary Address register for Secure Region */
    UH    reserved48;
    UH    VBRPABARNS;             /* VBATT Backup register Privilege Attribute Boundary */
    UH    reserved49;
    UW    reserved50;
    UW    CGFSAR;                 /* Clock Generation Function Security Attribute register */
    UW    RSTSAR;                 /* Reset Security Attribution register       */
    UW    LPMSAR;                 /* Low Power Mode Security Attribution register */
    UW    LVDSAR;                 /* Programable Voltage Detection Security Attribution register */
    UW    BBFSAR;                 /* Battery Backup Function Security Attribute register */
    UW    reserved51;
    UW    PGCSAR;                 /* Power Gating Control Security Attribution register */
    UW    reserved52;
    UW    DPFSAR;                 /* Deep Standby Interrupt Factor Security Attribution register */
    UW    RSCSAR;                 /* RAM Standby Control Security Attribution register   */
    UW    reserved53[4];
    UH    reserved54;
    UH    PRCR;                   /* Protect register for Secure register      */
    UH    reserved55;
    UH    PRCR_NS;                /* Protect register for Non-secure register  */
    UB    LOCOCR;                 /* Low-Speed On-Chip Oscillator Control register       */
    UB    reserved56;
    UB    LOCOUTCR;               /* LOCO User Trimming Control register       */
    UB    reserved57;
    UW    reserved58[2];
    UH    reserved59;
    UB    reserved60;
    UB    STCONR;                 /* Standby Condition register                */
    UW    reserved61[380];
    UB    DPSBYCR;                /* Deep Standby Control register             */
    UB    reserved62;
    UH    reserved63;
    UB    DPSWCR;                 /* Deep Standby Wait Control register        */
    UB    reserved64;
    UH    reserved65;
    UB    DPSIER0;                /* Deep Standby Interrupt Enable register 0  */
    UB    reserved66;
    UH    reserved67;
    UB    DPSIER1;                /* Deep Standby Interrupt Enable register 1  */
    UB    reserved68;
    UH    reserved69;
    UB    DPSIER2;                /* Deep Standby Interrupt Enable register 2  */
    UB    reserved70;
    UH    reserved71;
    UB    DPSIER3;                /* Deep Standby Interrupt Enable register 3  */
    UB    reserved72;
    UH    reserved73;
    UB    DPSIFR0;                /* Deep Standby Interrupt Flag register 0    */
    UB    reserved74;
    UH    reserved75;
    UB    DPSIFR1;                /* Deep Standby Interrupt Flag register 1    */
    UB    reserved76;
    UH    reserved77;
    UB    DPSIFR2;                /* Deep Standby Interrupt Flag register 2    */
    UB    reserved78;
    UH    reserved79;
    UB    DPSIFR3;                /* Deep Standby Interrupt Flag register 3    */
    UB    reserved80;
    UH    reserved81;
    UB    DPSIEGR0;               /* Deep Standby Interrupt Edge register 0    */
    UB    reserved82;
    UH    reserved83;
    UB    DPSIEGR1;               /* Deep Standby Interrupt Edge register 1    */
    UB    reserved84;
    UH    reserved85;
    UB    DPSIEGR2;               /* Deep Standby Interrupt Edge register 2    */
    UB    reserved86;
    UH    reserved87;
    UW    reserved88;
    UB    SYOCDCR;                /* System Control OCD Control register       */
    UB    reserved89;
    UH    reserved90;
    UW    reserved91;
    UB    RSTSR0;                 /* Reset Status register 0                   */
    UB    reserved92;
    UH    reserved93;
    UB    RSTSR2;                 /* Reset Status register 2                   */
    UB    reserved94;
    UH    reserved95;
    UB    RSTSR3;                 /* Reset Status register 3                   */
    UB    reserved96;
    UH    reserved97;
    UW    reserved98;
    UB    MOMCR;                  /* Main Clock Oscillator Mode Oscillation Control */
    UB    reserved99;
    UH    reserved100;
    UB    FWEPROR;                /* Flash Write Erase Protect register        */
    UB    reserved101;
    UH    reserved102;
    union {
        UB  LVCMPCR;              /* Voltage Monitor Circuit Control register  */
        UB  LVD1CMPCR;            /* Voltage Monitor 1 Comparator Control register */
    };
    UB    reserved103;
    UH    reserved104;
    UB    LVD2CMPCR;              /* Voltage Monitor 2 Comparator Control register */
    UB    reserved105;
    UH    reserved106;
    UW    reserved107[4];
    UB    LVD1CR0;                /* Voltage Monitor 1 Circuit Control register 0 */
    UB    reserved108;
    UH    reserved109;
    UB    LVD2CR0;                /* Voltage Monitor 2 Circuit Control register 0 */
    UB    reserved110;
    UH    reserved111;
    UW    reserved112[3];
    UB    VBATTMNSELR;            /* Battery Backup Voltage Monitor Function Select register */
    UB    reserved113;
    UH    reserved114;
    UB    VBTBPCR1;               /* VBATT Battery Power Supply Control register 1 */
    UB    reserved115;
    UH    reserved116;
    UW    reserved117;
    UB    LPSCR;                  /* Low Power State Control register          */
    UB    reserved118;
    UH    reserved119;
    UW    reserved120;
    UB    SSCR1;                  /* Software Standby Control register 1       */
    UB    reserved121;
    UH    reserved122;
    UW    reserved123[5];
    UB    LVOCR;                  /* Low Power State Control register          */
    UB    reserved124;
    UH    reserved125;
    UW    reserved126[7];
    UB    SYRSTMSK0;              /* System Reset Mask Control register0       */
    UB    reserved127;
    UH    reserved128;
    UB    SYRSTMSK1;              /* System Reset Mask Control register1       */
    UB    reserved129;
    UH    reserved130;
    UB    SYRSTMSK2;              /* System Reset Mask Control register2       */
    UB    reserved131;
    UH    reserved132;
    UW    reserved133[10];
    UB    PLL1LDOCR;              /* PLL1-LDO Control register                 */
    UB    reserved134;
    UH    reserved135;
    UB    PLL2LDOCR;              /* PLL2-LDO Control register                 */
    UB    reserved136;
    UH    reserved137;
    UB    HOCOLDOCR;              /* HOCO-LDO Control register                 */
    UB    reserved138;
    UH    reserved139;
    UW    reserved140[4];
    UB    LVD1FCR;                /* Voltage Monitor % Function Control register */
    UB    reserved141;
    UH    reserved142;
    UB    LVD2FCR;                /* Voltage Monitor % Function Control register */
    UB    reserved143;
    UH    reserved144;
    UW    reserved145[54];
    UB    SOSCCR;                 /* Sub-clock oscillator control register     */
    UB    SOMCR;                  /* Sub Clock Oscillator Mode Control register */
    UH    reserved146;
    UW    reserved147[15];
    UB    VBTBER;                 /* VBATT Backup Enable register              */
    UB    reserved148;
    UH    reserved149;
    UB    reserved150;
    UB    VBTBPCR2;               /* VBATT Battery Power Supply Control register 2 */
    UB    VBTBPSR;                /* VBATT Battery Power Supply Status register */
    UB    reserved151;
    UB    VBTADSR;                /* VBATT Tamper detection Status register    */
    UB    VBTADCR1;               /* VBATT Tamper detection Control register 1 */
    UB    VBTADCR2;               /* VBATT Tamper detection Control register 2 */
    UB    reserved152;
    UB    VBTICTLR;               /* VBATT Input Control register              */
    UB    VBTICTLR2;              /* VBATT Input Control register 2            */
    UB    VBTIMONR;               /* VBATT Input Monitor register              */
    UB    reserved153;
    UW    reserved154[44];
    UB    VBTBKR[128];            /* VBATT Backup register 0-127               */
} T_RA8_SYSC; 


/* Port access structure */

#ifdef __BUILD_SECURE__
#define REG_SYSC    (*(volatile T_RA8_SYSC *)SYSC_BASE)
#else
#define REG_SYSC    (*(volatile T_RA8_SYSC *)(SYSC_BASE | NS_BASE_OFFSET))
#endif


#ifdef __cplusplus
}
#endif
#endif /* _DV_RA8_SYSC_H_ */
