/***********************************************************************
    MULti Core Operating System - MultiLayer kenel (MULCOS-MK)

          Option setting memory for EK-RA8M1 (Setting via debugger)

    Copyright (c) 2024 SPIRAL-TECH Co.,ltd All Rights Reserved.

    File name   option_setting.c

    Sep/05/2024 Created.

 ***********************************************************************/


#include "kernel.h"


/***********************************************************************
    Definitions of macro parameter
 ***********************************************************************/

/* No Operation */
#define NO_OPRATION           0xFFFFFFFFul

/* OFS0 : Option Function Select Register 0 */
#define OFS0_INVBIT           0xA001A001ul
#define OFS0_WDT0STPCTL_SH    30
#define OFS0_WDT0RSTIRQS_SH   28
#define OFS0_WDT0PRSS_SH      26
#define OFS0_WDT0PRES_SH      24
#define OFS0_WDT0CKS_SH       20
#define OFS0_WDT0TOPS_SH      18
#define OFS0_WDT0STRT_SH      17
#define OFS0_IWDTSTPCTL_SH    14
#define OFS0_IWDTRSTIRQS_SH   12
#define OFS0_IWDTPRSS_SH      10
#define OFS0_IWDTPRES_SH      8
#define OFS0_IWDTCKS_SH       4
#define OFS0_IWDTTOPS_SH      2
#define OFS0_IWDTSTRT_SH      1

/* OFS2 : Option Function Select Register 2 */
#define OFS2_INVBIT           0xFFFFFFFEul
#define OFS2_DCDCEN_SH        0

/* DUALSEL : Dual Mode Select Register */
#define DUALSEL_INVBIT        0xFFFFFFF8ul
#define DUALSEL_BANKMD_SH     0

/* SAS : Startup Area Setting Register */
#define SAS_INVBIT            0x7FFF7FFFul
#define SAS_BTFLG_SH          31
#define SAS_FSPR_SH           15

/* OFS1, OFS1_SEC, OFS1_SEL : Option Function Select Register 1 */
#define OFS1_INVBIT           0xFCFFF0D0ul
#define OFS1_INITECCEN_SH     25
#define OFS1_SWDBG_SH         24
#define OFS1_HOCOFRQ0_SH      9
#define OFS1_HOCOEN_SH        8
#define OFS1_PVDLPSEL_SH      5
#define OFS1_PVDAS_SH         3
#define OFS1_VDSEL_SH         0

/* BANKSEL, BANKSEL_SEC : Bank Select Register */
#define BANKSEL_INVBIT        0xFF80FFF8ul
#define BANKSEL_BLCKSWP_SH    16
#define BANKSEL_BANKSWP_SH    0

/* Bank mode */
#define BANKMD_DUAL           0
#define BANKMD_LINEAR         7

/* Startup Setting */
#define BTFG_EXCHGL           0u
#define BTFG_NORMAL           1u
#define FSPR_INVALID          0u
#define FSPR_VALID            1u

/* DCDC enable */
#define DCDCEN_DISABLE        0u
#define DCDCEN_ENABALE        1u

/* Watchdog Timer Setting */
#define IWDTSTPCTL_CONTI      0u
#define IWDTSTPCTL_STOP       1u
#define IWDTRSTIRQS_INTR      0u
#define IWDTRSTIRQS_RESET     1u
#define IWDTRPSS_25           0u
#define IWDTRPSS_50           1u
#define IWDTRPSS_75           2u
#define IWDTRPSS_100          3u
#define IWDTRPES_75           0u
#define IWDTRPES_50           1u
#define IWDTRPES_25           2u
#define IWDTRPES_0            3u
#define IWDTCKS_DIV1          0u
#define IWDTCKS_DIV16         2u
#define IWDTCKS_DIV32         3u
#define IWDTCKS_DIV64         4u
#define IWDTCKS_DIV128        15u
#define IWDTCKS_DIV256        5u
#define IWDTTOPS_128          0u
#define IWDTTOPS_512          1u
#define IWDTTOPS_1024         2u
#define IWDTTOPS_2048         3u
#define IWDTSTRT_AUTO         0u
#define IWDTSTRT_MANU         1u

/* Watchdog Timer Setting */
#define WDT0STPCTL_CONTI      0u
#define WDT0STPCTL_STOP       1u
#define WDT0RSTIRQS_INTR      0u
#define WDT0RSTIRQS_RESET     1u
#define WDT0RPSS_25           0u
#define WDT0RPSS_50           1u
#define WDT0RPSS_75           2u
#define WDT0RPSS_100          3u
#define WDT0RPES_75           0u
#define WDT0RPES_50           1u
#define WDT0RPES_25           2u
#define WDT0RPES_0            3u
#define WDT0CKS_DIV4          1u
#define WDT0CKS_DIV64         4u
#define WDT0CKS_DIV128        15u
#define WDT0CKS_DIV512        6u
#define WDT0CKS_DIV2048       7u
#define WDT0CKS_DIV8192       8u
#define WDT0TOPS_1024         0u
#define WDT0TOPS_4096         1u
#define WDT0TOPS_8192         2u
#define WDT0TOPS_16384        3u
#define WDT0STRT_AUTO         0u
#define WDT0STRT_MANU         1u

/* Bank Swap */
#define BANKSWP_200000        0u
#define BANKSWP_000000        7u
#define BLCKSWP_PATTERN       0x7Fu

/* Startup configuration setting */
#define INITECCEN_DISABLE     0u
#define INITECCEN_ENABLE      1u
#define SWDBG_ENABLE          0u
#define SWDBG_DISABLE         1u
#define HOCOFRQ0_16M          0u
#define HOCOFRQ0_18M          1u
#define HOCOFRQ0_20M          2u
#define HOCOFRQ0_32M          4u
#define HOCOFRQ0_48M          7u
#define HOCOEN_ENABLE         0u
#define HOCOEN_DISABLE        1u
#define PVDLPSEL_ENABLE       0u
#define PVDLPSEL_DISABLE      1u
#define PVDAS_RST_ENABLE      0u
#define PVDAS_RST_DISABLE     1u
#define VDSEL_2R85            0u
#define VDSEL_2R58            1u
#define VDSEL_2R15            2u
#define VDSEL_2R00            3u
#define VDSEL_1R90            4u
#define VDSEL_1R80            5u
#define VDSEL_1R70            6u
#define VDSEL_1R60            7u

/* FSBLCTRL0 : FSBL Control Register 0 */
#define FSBLCTRL0_INVBIT      0xFFFFF000u
#define FSBLCTRL0_FSBLCLK_SH  9
#define FSBLCTRL0_FSBLSKIPDS_SH 6
#define FSBLCTRL0_FSBLSKIPSW_SH 3
#define FSBLCTRL0_FSBLEN_SH   0

/* FSBLCTRL0 : FSBL Control Register 1 */
#define FSBLCTRL1_INVBIT      0xFFFFFFFCu
#define FSBLCTRL1_FSBLEXMD_SH 0

/* FSBLCTRL0 : FSBL Control Register 2 */
#define FSBLCTRL2_INVBIT      0xFFFFFE00u
#define FSBLCTRL2_PORTGN_SH   4
#define FSBLCTRL2_PORTPN_SH   0

/* FSBL Enable */
#define FSBLCLK_120MH         0
#define FSBLCLK_240MH         7
#define FSBLSKIPDS_ENABLE     0
#define FSBLSKIPDS_DISABLE    7
#define FSBLSKIPSW_ENABLE     0
#define FSBLSKIPSW_DISABLE    7
#define FSBLEN_ENABLE         0
#define FSBLEN_DISABLE        7

/* FSBL Boot Mode */
#define FSBLEXMD_CRC          0u
#define FSBLEXMD_CRC_NOTICE   1u
#define FSBLEXMD_SEC          2u
#define FSBLEXMD_SEC_NOTICE   3u

/* FSBL Error Notice Port */
#define PORTPN(m)             (m)
#define PORTGR(n)             (n)


/***********************************************************************
    Definitions of parameter
 ***********************************************************************/

#if defined(__BUILD_SECURE__)
#define OPTION_SETTING_OFS0         ( OFS0_INVBIT                               | \
                                     (WDT0STPCTL_STOP   << OFS0_WDT0STPCTL_SH)  | \
                                     (WDT0RSTIRQS_RESET << OFS0_WDT0RSTIRQS_SH) | \
                                     (WDT0RPSS_100      << OFS0_WDT0PRSS_SH)    | \
                                     (WDT0RPES_0        << OFS0_WDT0PRES_SH)    | \
                                     (WDT0CKS_DIV128    << OFS0_WDT0CKS_SH)     | \
                                     (WDT0TOPS_16384    << OFS0_WDT0TOPS_SH)    | \
                                     (WDT0STRT_MANU     << OFS0_WDT0STRT_SH)    | \
                                     (IWDTSTPCTL_STOP   << OFS0_IWDTSTPCTL_SH)  | \
                                     (IWDTRSTIRQS_RESET << OFS0_IWDTRSTIRQS_SH) | \
                                     (IWDTRPSS_100      << OFS0_IWDTPRSS_SH)    | \
                                     (IWDTRPES_0        << OFS0_IWDTPRES_SH)    | \
                                     (IWDTCKS_DIV128    << OFS0_IWDTCKS_SH)     | \
                                     (IWDTTOPS_2048     << OFS0_IWDTTOPS_SH)    | \
                                     (IWDTSTRT_MANU     << OFS0_IWDTSTRT_SH))
#define OPTION_SETTING_OFS2         ( OFS2_INVBIT                               | \
                                     (DCDCEN_ENABALE    << OFS2_DCDCEN_SH))
#define OPTION_SETTING_DUALSEL      ( DUALSEL_INVBIT                            | \
                                     (BANKMD_LINEAR     << DUALSEL_BANKMD_SH))
//                                     (BANKMD_DUAL       << DUALSEL_BANKMD_SH))
#define OPTION_SETTING_SAS          ( SAS_INVBIT                                | \
                                     (BTFG_NORMAL       << SAS_BTFLG_SH)        | \
                                     (FSPR_VALID        << SAS_FSPR_SH))

#define OPTION_SETTING_OFS1_SEC     ( OFS1_INVBIT                               | \
                                     (VDSEL_1R60        << OFS1_VDSEL_SH)       | \
                                     (PVDAS_RST_DISABLE << OFS1_PVDAS_SH)       | \
                                     (PVDLPSEL_DISABLE  << OFS1_PVDLPSEL_SH)    | \
                                     (HOCOEN_DISABLE    << OFS1_HOCOEN_SH)      | \
                                     (HOCOFRQ0_48M      << OFS1_HOCOFRQ0_SH)    | \
                                     (SWDBG_DISABLE     << OFS1_SWDBG_SH)       | \
                                     (INITECCEN_DISABLE << OFS1_INITECCEN_SH))
#define OPTION_SETTING_BANKSEL_SEC  ( BANKSEL_INVBIT                            | \
                                     (BLCKSWP_PATTERN   << BANKSEL_BLCKSWP_SH)  | \
                                     (BANKSWP_000000    << BANKSEL_BANKSWP_SH))
#define OPTION_SETTING_BPS_SEC0     (NO_OPRATION)
#define OPTION_SETTING_BPS_SEC1     (NO_OPRATION)
#define OPTION_SETTING_BPS_SEC2     (NO_OPRATION)
#define OPTION_SETTING_BPS_SEC3     (NO_OPRATION)
#define OPTION_SETTING_PBPS_SEC0    (NO_OPRATION)
#define OPTION_SETTING_PBPS_SEC1    (NO_OPRATION)
#define OPTION_SETTING_PBPS_SEC2    (NO_OPRATION)

#define OPTION_SETTING_OFS1_SEL     ((HOCOFRQ0_16M       << OFS1_HOCOFRQ0_SH)   | \
                                     (HOCOFRQ0_18M       << OFS1_HOCOFRQ0_SH)   | \
                                     (HOCOFRQ0_20M       << OFS1_HOCOFRQ0_SH)   | \
                                     (HOCOFRQ0_32M       << OFS1_HOCOFRQ0_SH)   | \
                                     (HOCOFRQ0_48M       << OFS1_HOCOFRQ0_SH)   | \
                                     (HOCOEN_DISABLE     << OFS1_HOCOEN_SH)     | \
                                     (HOCOEN_ENABLE      << OFS1_HOCOEN_SH))

#define OPTION_SETTING_BANKSEL_SEL  (NO_OPRATION)
#if 1
#define OPTION_SETTING_BPS_SEC0_SEL (NO_OPRATION)
#define OPTION_SETTING_BPS_SEC1_SEL (NO_OPRATION)
#define OPTION_SETTING_BPS_SEC2_SEL (NO_OPRATION)
#define OPTION_SETTING_BPS_SEC3_SEL (NO_OPRATION)
#else
#define OPTION_SETTING_BPS_SEC0_SEL (0x00000000u)
#define OPTION_SETTING_BPS_SEC1_SEL (0xFFFFFFC0u)
#define OPTION_SETTING_BPS_SEC2_SEL (0x0000001Fu)
#define OPTION_SETTING_BPS_SEC3_SEL (0x00000000u)
#endif


#define OPTION_SETTING_FSBLCTRL0    ( FSBLCTRL0_INVBIT                          | \
                                     (FSBLCLK_240MH      << FSBLCTRL0_FSBLCLK_SH)    | \
                                     (FSBLSKIPDS_DISABLE << FSBLCTRL0_FSBLSKIPDS_SH) | \
                                     (FSBLSKIPSW_DISABLE << FSBLCTRL0_FSBLSKIPSW_SH) | \
                                     (FSBLEN_DISABLE     << FSBLCTRL0_FSBLEN_SH))
#define OPTION_SETTING_FSBLCTRL1    ( FSBLCTRL1_INVBIT                          | \
                                     (FSBLEXMD_SEC_NOTICE << FSBLCTRL1_FSBLEXMD_SH))
#define OPTION_SETTING_FSBLCTRL2    ( FSBLCTRL2_INVBIT                          | \
                                     (PORTPN(15))        << FSBLCTRL2_PORTPN_SH | \
                                     (PORTGR(31)         << FSBLCTRL2_PORTGN_SH))
#define OPTION_SETTING_SACC0        (NO_OPRATION)
#define OPTION_SETTING_SACC1        (NO_OPRATION)
#define OPTION_SETTING_SAMR         (NO_OPRATION)

#else
#define OPTION_SETTING_OFS1         ( OFS1_INVBIT                               | \
                                     (VDSEL_1R60        << OFS1_VDSEL_SH)       | \
                                     (PVDAS_RST_DISABLE << OFS1_PVDAS_SH)       | \
                                     (PVDLPSEL_DISABLE  << OFS1_PVDLPSEL_SH)    | \
                                     (HOCOEN_DISABLE    << OFS1_HOCOEN_SH)      | \
                                     (HOCOFRQ0_48M      << OFS1_HOCOFRQ0_SH)    | \
                                     (SWDBG_DISABLE     << OFS1_SWDBG_SH)       | \
                                     (INITECCEN_DISABLE << OFS1_INITECCEN_SH))
#define OPTION_SETTING_BANKSEL      ( BANKSEL_INVBIT                            | \
                                     (BLCKSWP_PATTERN   << BANKSEL_BLCKSWP_SH)  | \
                                     (BANKSWP_000000    << BANKSEL_BANKSWP_SH))
#define OPTION_SETTING_BPS0         (NO_OPRATION)
#define OPTION_SETTING_BPS1         (NO_OPRATION)
#define OPTION_SETTING_BPS2         (NO_OPRATION)
#define OPTION_SETTING_BPS3         (NO_OPRATION)
#define OPTION_SETTING_PBPS0        (NO_OPRATION)
#define OPTION_SETTING_PBPS1        (NO_OPRATION)
#define OPTION_SETTING_PBPS2        (NO_OPRATION)
#endif


/***********************************************************************
    Definitions of configuration data
 ***********************************************************************/

#ifdef __GNUC__

extern const UW __tz_FLASH_N;
extern const UW __FLASH_NSC_START;
extern const UW __tz_DATA_FLASH_N;
extern const UW __tz_RAM_N;
extern const UW __RAM_NSC_START;

#if defined(__BUILD_SECURE__)
const UW __attribute__((section(".option_setting_ofs0")))         g_ofs0         = OPTION_SETTING_OFS0;
const UW __attribute__((section(".option_setting_ofs2")))         g_ofs2         = OPTION_SETTING_OFS2;
const UW __attribute__((section(".option_setting_dualsel")))      g_dualsel      = OPTION_SETTING_DUALSEL;
const UW __attribute__((section(".option_setting_sas")))          g_sas          = OPTION_SETTING_SAS;

const UW __attribute__((section(".option_setting_ofs1_sec")))     g_ofs1_sec     = OPTION_SETTING_OFS1_SEC;
const UW __attribute__((section(".option_setting_banksel_sec")))  g_banksel      = OPTION_SETTING_BANKSEL_SEC;
const UW __attribute__((section(".option_setting_bps_sec0")))     g_bps_sec0     = OPTION_SETTING_BPS_SEC0;
const UW __attribute__((section(".option_setting_bps_sec1")))     g_bps_sec1     = OPTION_SETTING_BPS_SEC1;
const UW __attribute__((section(".option_setting_bps_sec2")))     g_bps_sec2     = OPTION_SETTING_BPS_SEC2;
const UW __attribute__((section(".option_setting_bps_sec3")))     g_bps_sec3     = OPTION_SETTING_BPS_SEC3;
const UW __attribute__((section(".option_setting_pbps_sec0")))    g_pbps_sec0    = OPTION_SETTING_PBPS_SEC0;
const UW __attribute__((section(".option_setting_pbps_sec1")))    g_pbps_sec1    = OPTION_SETTING_PBPS_SEC1;
const UW __attribute__((section(".option_setting_pbps_sec2")))    g_pbps_sec2    = OPTION_SETTING_PBPS_SEC2;

const UW __attribute__((section(".option_setting_ofs1_sel")))     g_ofs1_sel     = OPTION_SETTING_OFS1_SEL;
const UW __attribute__((section(".option_setting_banksel_sel")))  g_banksel_sel  = OPTION_SETTING_BANKSEL_SEL;
const UW __attribute__((section(".option_setting_bps_sec0_sel"))) g_bps_sec0_sel = OPTION_SETTING_BPS_SEC0_SEL;
const UW __attribute__((section(".option_setting_bps_sec1_sel"))) g_bps_sec1_sel = OPTION_SETTING_BPS_SEC1_SEL;
const UW __attribute__((section(".option_setting_bps_sec2_sel"))) g_bps_sec2_sel = OPTION_SETTING_BPS_SEC2_SEL;
const UW __attribute__((section(".option_setting_bps_sec3_sel"))) g_bps_sec3_sel = OPTION_SETTING_BPS_SEC3_SEL;

const UW __attribute__((section(".option_setting_fsblctrl0")))    g_fsblctrl0    = OPTION_SETTING_FSBLCTRL0;
const UW __attribute__((section(".option_setting_fsblctrl1")))    g_fsblctrl1    = OPTION_SETTING_FSBLCTRL1;
const UW __attribute__((section(".option_setting_fsblctrl2")))    g_fsblctrl2    = OPTION_SETTING_FSBLCTRL2;
const UW __attribute__((section(".option_setting_sacc0")))        g_sacc0        = OPTION_SETTING_SACC0;
const UW __attribute__((section(".option_setting_sacc1")))        g_sacc1        = OPTION_SETTING_SACC1;
const UW __attribute__((section(".option_setting_samr")))         g_samr         = OPTION_SETTING_SAMR;
#else
const UW __attribute__((section(".option_setting_ofs1")))         g_ofs1_sec     = OPTION_SETTING_OFS1;
const UW __attribute__((section(".option_setting_banksel")))      g_banksel      = OPTION_SETTING_BANKSEL;
const UW __attribute__((section(".option_setting_bps0")))         g_bps_sec0     = OPTION_SETTING_BPS0;
const UW __attribute__((section(".option_setting_bps1")))         g_bps_sec1     = OPTION_SETTING_BPS1;
const UW __attribute__((section(".option_setting_bps2")))         g_bps_sec2     = OPTION_SETTING_BPS2;
const UW __attribute__((section(".option_setting_bps3")))         g_bps_sec3     = OPTION_SETTING_BPS3;
const UW __attribute__((section(".option_setting_pbps0")))        g_pbps_sec0    = OPTION_SETTING_PBPS0;
const UW __attribute__((section(".option_setting_pbps1")))        g_pbps_sec1    = OPTION_SETTING_PBPS1;
const UW __attribute__((section(".option_setting_pbps2")))        g_pbps_sec2    = OPTION_SETTING_PBPS2;
#endif


#endif


/* end */
