/***********************************************************************
 *  MULti Core Operating System - Multilayer Kernel (MULCOS-MK)

          First initializing for EK-RA8M1

    Copyright (c) 2024 SPIRAL-TECH Co.,ltd All Rights Reserved.

    File name   sysinit.c

    Sep/14/2024 Created.

 ***********************************************************************/

#include "kernel.h"
#include "arch_device.h"
#include "dv_ra8m1.h"


/******************************************************************
    Prototype definitions
 ******************************************************************/

void _init_clock_device(void);
void init_mpu(void);


/******************************************************************
    Configuration MPU constant data
 ******************************************************************/

#define MPU_ATTIR0  (MPU_ATR_nGnRnE)                    /* Device Attribute */
#define MPU_ATTIR1  (MPU_ATR_OutNon | MPU_ATR_InnNon)   /* NonCache Attribute */
#define MPU_ATTIR2  (MPU_ATR_OutNon | MPU_ATR_InnAWT)   /* WriteThrough Attribute */
#define MPU_ATTIR3  (MPU_ATR_OutNon | MPU_ATR_InnAWBAR) /* WriteBack    Attribute */
#define MPU_ATTIR4  (MPU_ATR_nGnRnE)                    /* Unused */
#define MPU_ATTIR5  (MPU_ATR_nGnRnE)                    /* Unused */
#define MPU_ATTIR6  (MPU_ATR_nGnRnE)                    /* Unused */
#define MPU_ATTIR7  (MPU_ATR_nGnRnE)                    /* Unused */


const T_MPUTBL  mpu_table[] = {
      /* Code */
    { 0x12100000u | MPU_BIT_NonShare | MPU_BIT_UnPriRO,
      0x121F7FE0u | MPU_ATTIR(2)     | MPU_BIT_Enable},
      /* User's Data */
    { 0x32060000u | MPU_BIT_NonShare | MPU_BIT_UnPriRW,
      0x320DDFE0u | MPU_ATTIR(2)     | MPU_BIT_Enable},
      /* System Data (only privilege mode) */
    { 0x320DE000u | MPU_BIT_NonShare | MPU_BIT_PriRW,
      0x320DFFE0u | MPU_ATTIR(2)     | MPU_BIT_Enable},
      /* Periphera */
    { 0x50000000u | MPU_BIT_NonShare | MPU_BIT_UnPriRW,
      0x5FFFFFE0u | MPU_ATTIR(0)     | MPU_BIT_Enable}};


/******************************************************************
    Initialize system device
 ******************************************************************/

void _init_clock_device(void)
{
    /* Unlock protect */
    REG_SYSC.PRCR = 0xA503;

    /* Lock protect */
    REG_SYSC.PRCR = 0xA500;

}


/******************************************************************
    Initialize Memory Protection Unit (MPU)
 ******************************************************************/

void init_mpu(void)
{
    INT max;
    INT i, j;

    max = (MPU_TYPE & 0x0000FF00u) >> 8;
    for(i = 0; i < max; i++) {
        MPU_RNR = (UW)i;
        MPU_RBAR = 0x00000000u;
        MPU_RLAR = MPU_BIT_Disable;
    }

    j = sizeof(mpu_table) / sizeof(T_MPUTBL);
    if (j > max) {
        j = max;
    }
    for(i = 0; i < j; i++) {
        MPU_RNR = (UW)i;
        MPU_RBAR = mpu_table[i].rbar;
        MPU_RLAR = mpu_table[i].rlar;
    }
    MPU_MAIR0 = (MPU_ATTIR3 << 24) | (MPU_ATTIR2 << 16) |
                (MPU_ATTIR1 <<  8) | (MPU_ATTIR0 <<  0);
    MPU_MAIR1 = (MPU_ATTIR7 << 24) | (MPU_ATTIR6 << 16) |
                (MPU_ATTIR5 <<  8) | (MPU_ATTIR4 <<  0);
    MPU_CTRL = 0x00000001;
    __DSB();
    /* Set CCR.DC field */
    SYSR_CCR |= 0x00010000;
    __DSB();
    __ISB();
}

/* end */
