/*
 *
 *  MULti Core Operating System Tiny Kernel (MULCOS-T)
 *                      Device driver for RA8 ICU
 *
 *           Copyright (c) 2025 SPIRAL-TECH Co.,ltd All Rights Reserved.
 *
 *  File name       drv_ra8_icu.c
 *
 *  Sep/19/2024     Created.
 *
 */

#include "kernel.h"
#include "arch_device.h"
#include "drv_ra8_icu.h"
#include "drv_icu_cfg.h"


/*
 *      Definitions of Management constant
 */

//#define EVENT_MAX     0x19C


/*
 *      Allocate the exception ID from the event number
 *      CxgԍOԍ擾ĕRt
 *          16ȏ ... tꂽOԍ
 *          E_NOID ... Oԍ̋󂫂
 *          E_OBJ .... ɗOԍtĂ
 *          E_PAR .... Cxgԍs
 */

ER_ID allocate_excp_id(ID event_no, UW attr)
{
    volatile UW *icusar;
    BOOL state;
    UW actbit;
    INT inttype;
    INT i;
    ID excp_id = 0;

    if ((event_no > 0) && (event_no <= EVENT_MAX) &&
        ((attr & ~IELSR_DTCE) == 0)) {
        state = sns_loc();
        if (state == FALSE) {
            loc_cpu();
        }
        inttype = (NVIC_ICTR & 0xFu) + 1;
        icusar = &REG_CPSCU.ICUSARG;
        for(i = 0; i < (inttype * 32); i++) {
#if defined(__BUILD_SECURE__)
            actbit = ~icusar[i / 32];
#else
            actbit = icusar[i / 32];
#endif
            if (actbit != 0) {
                if ((actbit & (0x00000001u << (i % 32))) != 0) {
                    if ((REG_ICU.IELSR[i] & 0x0000001Fu) == (UW)event_no) {
                        excp_id = E_OBJ;
                        break;
                    }
                }
                actbit &= ~(0x00000001u << (i % 32));
                __DSB();
            } else {
                i = ((i / 32) * 32) + 31;
            }
        }
        if (excp_id == 0) {
            for(i = 0; i < (inttype * 32); i++) {
#if defined(__BUILD_SECURE__)
                actbit = ~icusar[i / 32];
#else
                actbit = icusar[i / 32];
#endif
                if (actbit != 0) {
                    if ((actbit & (0x00000001u << (i % 32))) != 0) {
                        if (REG_ICU.IELSR[i] == 0) {
                            REG_ICU.IELSR[i] = (UW)event_no | attr;
                            __DSB();
                            excp_id = i + 16;
                            break;
                        }
                    }
                }
            }
            if (excp_id == 0) {
                excp_id = E_NOID;
            }
        }
        if (state == FALSE) {
            unl_cpu();
        }
    } else {
        excp_id = E_PAR;
    }
    return excp_id;
}


/*
 *      Get the exception ID from the event number
 *      CxgԍɕRtꂽOԍ擾
 *          0 ........ tꂽOԍ
 *          0ȏ .... tĂOԍ
 *          E_OBJ .... ZLA/ZLAsv
 *          E_PAR .... Cxgԍs
 */

ER_ID get_excp_id(ID event_no)
{
    volatile UW *icusar;
    BOOL state;
    UW actbit;
    W inttype;
    W i;
    ER_ID excp_id = E_OBJ;

    if ((event_no > 0) && (event_no <= EVENT_MAX)) {
        state = sns_loc();
        if (state == FALSE) {
            loc_cpu();
        }
        inttype = (NVIC_ICTR & 0xFu) + 1;
        icusar = &REG_CPSCU.ICUSARG;
        for(i = 0; i < (inttype * 32); i++) {
#if defined(__BUILD_SECURE__)
            actbit = ~icusar[i / 32];
#else
            actbit = icusar[i / 32];
#endif
            if (actbit != 0) {
                if ((actbit & (0x00000001u << (i % 32))) != 0) {
                    if ((REG_ICU.IELSR[i] & 0x0000001Fu) == (UW)event_no) {
                        excp_id = i + 16;
                        break;
                    }
                }
            } else {
                i = ((i / 32) * 32) + 31;
            }
        }
        if (state == FALSE) {
            unl_cpu();
        }
    } else {
        excp_id = E_PAR;
    }
    return excp_id;
}


/*
 *      Release the exception ID
 *      OԍRtĂCxgԍJ
 *          E_OK ..... ɊJ
 *          E_OBJ .... ZLA/ZLAsvA܂͊tĂȂ
 *          E_PAR .... Oԍs
 */

ER release_excp_id(ID excp_id)
{
    volatile UW *icusar;
    BOOL state;
    W inttype;
    ER retcd = E_OK;;

    state = sns_loc();
    if (state == FALSE) {
        loc_cpu();
    }
    inttype = (NVIC_ICTR & 0xFu) + 1;
    if ((excp_id >= 16) && (excp_id <= (inttype*32 + 16))) {
        icusar = &REG_CPSCU.ICUSARG;
#if defined(__BUILD_SECURE__)
        if ((icusar[(excp_id - 16) / 32] & (0x00000001u << ((excp_id - 16) % 32))) == 0) {
#else
        if ((icusar[(excp_id - 16) / 32] & (0x00000001u << ((excp_id - 16) % 32))) != 0) {
#endif
            if (REG_ICU.IELSR[excp_id - 16] != 0) {
                REG_ICU.IELSR[excp_id - 16] = 0;
                __DSB();
            } else {
                retcd = E_OBJ;
            }
        } else {
            retcd = E_OBJ;
        }
    } else {
        excp_id = E_PAR;
    }
    if (state == FALSE) {
        unl_cpu();
    }
    return retcd;
}
